const POST_INSTALL_URL = `http://www.zoominfo.com/b2b-reachout-extension-success`;
const URLS_BLACKLIST_KEY = "urlBlacklist";
const ZI_APP_FQDN = 'app.zoominfo.com';
let isRoReady = false;

let urlsBlacklist;

//If already in local storage, don't reinit.
//This is the "blacklist" of websites to NOT show RO on.
if (!localStorage.getItem(URLS_BLACKLIST_KEY)) {
    urlsBlacklist = {}

    localStorage.setItem(URLS_BLACKLIST_KEY, JSON.stringify(urlsBlacklist));
}

chrome.runtime.onInstalled.addListener((details) => {
    const TIMEOUT_MILLIS = 20000;
    const READY_CHECK_INTERVAL_MILLIS = 200;
    if (details.reason === 'install') {
        const postInstallRootDomain = extractRootDomain(POST_INSTALL_URL);
        setUrlPermissions(postInstallRootDomain, true);
        window.open(POST_INSTALL_URL);
        let intervalDuration = 0;
        chrome.tabs.getSelected(null, function (tab) {
            const checkRoReadyInterval = setInterval(function () {
                intervalDuration += READY_CHECK_INTERVAL_MILLIS;
                if (isRoReady) {
                    chrome.tabs.sendMessage(tab.id, {type: 'open'});
                    clearInterval(checkRoReadyInterval);
                } else if (intervalDuration >= TIMEOUT_MILLIS) {
                    clearInterval(checkRoReadyInterval);
                }
            }, READY_CHECK_INTERVAL_MILLIS);
        });
    }
});

function createNotificationOptions(request) {
    let notificationsOptions = {
        iconUrl: "assets/images/48x48.png",
        type: "basic",
        title: request.options.title,
        contextMessage: request.options.contextMessage,
        message: request.options.message,
        buttons: []
    };
    if (navigator.appVersion.indexOf("Mac") !== -1) {
        request.options.buttons.mac.forEach( (btn) => {
            notificationsOptions.buttons.push(btn);
        });
    } else {
        request.options.buttons.windows.forEach( (btn) => {
            notificationsOptions.buttons.push(btn);
        });
    }
    return notificationsOptions;
}

function createRONotification(request) {
    return new Promise((resolve, reject) => {
        try {
            chrome.notifications.getAll(function(notificationsIds) {
                for (var id in notificationsIds) {
                    chrome.notifications.clear(id);
                }

                let notificationId = `ro-notification-${Date.now()}`;
                let notificationsOptions = createNotificationOptions(request);
                chrome.notifications.create(notificationId, notificationsOptions, function (res) {
                    if (res) {
                       chrome.tabs.query({active: true, currentWindow: true}, function (tabs) {
                            chrome.tabs.sendMessage(tabs[0].id, {
                                RONotificationWasDisplayed: true,
                                notificationId: notificationId
                            }, function (response) {
                            });
                           resolve();
                        });
                    } else {
                        console.error("error create notification on background.js :", chrome.runtime.lastError);
                        reject();
                    }
                });
            });
        }
       catch (err) {
           console.error("exception create notification on background.js :", chrome.runtime.lastError);
           reject();
       }
    });
}

function notificationHandleClickOnTheFirstBtn (notificationId) {
    if (navigator.appVersion.indexOf("Mac") !== -1) {
        chrome.tabs.query({active: true, currentWindow: true}, function (tabs) {
            chrome.tabs.sendMessage(tabs[0].id, {RONotificationSnoozeForTheCurrentDay: true, notificationId:notificationId}, function (response) {
                chrome.notifications.clear(notificationId);
            });
        });
    }
    else {
        chrome.tabs.query({active: true, currentWindow: true}, function (tabs) {
            chrome.tabs.sendMessage(tabs[0].id, {openRONotification: true, notificationId:notificationId}, function (response) {
                chrome.notifications.clear(notificationId);
            });
        });
    }
}

function notificationHandleClickOnTheSecondBtn (notificationId) {
    if (navigator.appVersion.indexOf("Mac") !== -1) {
        chrome.tabs.query({active: true, currentWindow: true}, function (tabs) {
            chrome.tabs.sendMessage(tabs[0].id, {RONotificationSnoozeForWeek: true, notificationId:notificationId}, function (response) {
                chrome.notifications.clear(notificationId);
            });
        });
    }
    else {
        chrome.tabs.query({active: true, currentWindow: true}, function (tabs) {
            chrome.tabs.sendMessage(tabs[0].id, {RONotificationSnoozeForTheCurrentDay: true, notificationId:notificationId}, function (response) {
                chrome.notifications.clear(notificationId);
            });
        });
    }
}

function addNotificationButtonsListeners() {
    if(chrome.notifications){
        chrome.notifications.onClicked.addListener(function (notificationId) {
            chrome.tabs.query({active: true, currentWindow: true}, function (tabs) {
                chrome.tabs.sendMessage(tabs[0].id, {openRONotification: true, notificationId:notificationId}, function (response) {
                    chrome.notifications.clear(notificationId);
                });
            });
        });

        chrome.notifications.onButtonClicked.addListener(function (notificationId, btnIdx) {
            if (btnIdx === 0) {
                notificationHandleClickOnTheFirstBtn(notificationId);
            } else if (btnIdx === 1) {
                notificationHandleClickOnTheSecondBtn(notificationId);
            }
        });
    }
}




chrome.runtime.onMessage.addListener(
    function (request, sender, sendResponse) {
        if (request.type === 'RONotification') {
           createRONotification(request).then(() =>{
                sendResponse({status: true});
            });
        } else {
            if (functions[request.func]) {
                if (!request.args) {
                    request.args = {};
                }
                request.args.sender = sender;
                functions[request.func].call(this, request.args).then((response) => {
                    sendResponse(response);
                });
            }
        }

        return true;
    });


/**
 * This handles if the browser icon was clicked.
 */
chrome.browserAction.onClicked.addListener(function (tab) {
    //If browser icon is clicked, we'll want to remove it from the blacklist.
    let rootDomain = extractRootDomain(tab.url);
    let extensionPermission = checkUrlPermissions(rootDomain);

    //Check extension permission. If false, then blacklist needs to be adjusted before proceeding.
    if (!extensionPermission) {

        //Remove from blacklist
        setUrlPermissions(rootDomain, true);

        //We know it was closed, so just send open message.
        chrome.tabs.sendMessage(tab.id, {type: 'open'});
    } else {
        //We know the extension exists, but we do not know the state. Send browser icon clicked message.
        //Let index handle it.
        chrome.tabs.sendMessage(tab.id, {type: 'browserIconClicked'})
    }
});

chrome.tabs.onUpdated.addListener(function (tabId, info, tab) {
    //Navigate to new URL
    if (info.status === 'loading' && info.url) {
        urlMap[tabId] = info.url;
    }
    //Refreshing page I'm already on.
    else if (info.status === 'loading' && !info.url) {
        urlMap[tabId] = tab.url;
    }

    if ((info.status === 'complete' && (urlMap[tabId] && urlMap[tabId] === tab.url))) {
        delete urlMap[tabId];
        chrome.tabs.sendMessage(tabId, {type: 'onUpdated', tabUrl: tab.url});
    }
});

let urlMap = {};
chrome.tabs.onRemoved.addListener(function (tabId, removeInfo) {
    if (urlMap[tabId]) {
        delete urlMap[tabId];
    }
});

let functions = {
    reload: reload,
    executeScript: executeScript,
    getLocalStorage: getLocalStorage,
    setLocalStorage: setLocalStorage,
    deleteFromLocalStorage: deleteFromLocalStorage,
    getCurrentTabUrl: getCurrentTabUrl,
    getCurrentWindow: getCurrentWindow,
    roReady: roReady,
    addUrlToBlacklist: addUrlToBlacklist,
    askForNotificationsPermission: askForNotificationsPermission,
    createContextMenus: createContextMenus
};


// TBD - change all callbacks to promises
function getCurrentWindow(event) {
    console.log("getCurrentWindow called for tab :: ", event.sender.tab);
    return new Promise((resolve, reject) => {
        let rootDomain = extractRootDomain(event.sender.tab.url);
        let isAllowed = checkUrlPermissions(rootDomain) && extractHostname(event.sender.tab.url) !== ZI_APP_FQDN;

        let returnObj = {
            window: null,
            isAllowed: true
        }

        chrome.windows.getCurrent(function (window) {
            returnObj.window = window;
            returnObj.isAllowed = isAllowed;
            resolve(returnObj);
        });
    });
}

function getCurrentTabUrl(args) {
    return new Promise((resolve, reject) => {
        return resolve(args.sender.tab.url);
    });
}

function getLocalStorage(args) {
    var name = args.name;
    return new Promise((resolve) => {
        return resolve(localStorage[name])
    });
}

function setLocalStorage(args) {
    var options = args.options;
    return new Promise((resolve) => {
        localStorage[options.name] = options.value;
        return resolve()
    });
}

function deleteFromLocalStorage(args) {
    var name = args.name;
    return new Promise((resolve) => {
        localStorage.removeItem(name);
        return resolve()
    });
}

function askForNotificationsPermission() {
    return new Promise((resolve, reject) => {
        try {
            chrome.permissions.request({
                permissions: ['notifications']
            }, function (granted) {
                if (granted) {
                    setLocalStorage({options : { name: 'notificationPermissionsAllowed', value: true}}).then(res => {
                        addNotificationButtonsListeners();
                        return resolve();
                    })
                } else {
                    setLocalStorage({options : { name: 'notificationPermissionsAllowed', value: false}}).then(res => {
                        return reject();
                    })
                }
            });
        }
        catch (err) {
            console.error("exception occurred while requesting permissions on background.js :", chrome.runtime.lastError);
            return reject();
        }
    });
}

function createContextMenus(menus) {
    return new Promise((resolve) => {
        chrome.contextMenus.removeAll();
        menus.forEach((menu) => {
            chrome.contextMenus.create(menu);
        });
        return resolve();
    });
}

chrome.contextMenus.onClicked.addListener((info, tab) => {
    chrome.tabs.sendMessage(tab.id, {type: 'onContextMenuClicked', searchType: info.menuItemId, searchQuery: info.selectionText})
});

function reload() {
    "use strict";
    chrome.tabs.getSelected(null, function (tab) {
        chrome.tabs.reload(tab.id);
    });
}

function executeScript(args) {
    return new Promise(function (resolve, reject) {
        chrome.tabs.executeScript(args.sender.tab.id, {
                file: args.path
            },
            function (response) {
                var lastError = getLastError();
                if (lastError) {
                    if (lastError.message === 'Cannot access a chrome:// URL' ||
                        lastError.message === 'The extensions gallery cannot be scripted.') {
                        console.log('There was an error injecting script : \n' + lastError.message);
                        return reject('You are at Chrome Extension Setting Page');
                    }
                    return reject({reason: 'COULD_NOT_IDENTIFY_PAGE'});
                }
                return resolve(response);
            });
    });
};

function getLastError() {
    return chrome.runtime.lastError;
}

/**
 * Closes the extension and adds current domain to the blacklist.
 *
 * @param event
 * @returns {Promise<any>}
 */
function addUrlToBlacklist(event) {
    return new Promise((resolve, reject) => {
        let tabUrl = extractRootDomain(event.sender.tab.url);
        setUrlPermissions(tabUrl, false);
        resolve();
    });
}

/**
 * Check the URL blacklist to determine if the extension is able to load.
 *
 * @param url - Root domain of the page currently being looked at.
 * @returns {boolean} - Whether the extension can load or not.
 */
function checkUrlPermissions(url) {
    let notAllowedUrls = JSON.parse(localStorage.getItem(URLS_BLACKLIST_KEY));
    if (url in notAllowedUrls) {
        return false;
    }
    return true;
}

/**
 * Add/Remove the URL from the URL blacklist.
 *
 * @param url - Root domain of the page currently being looked at.
 * @param value - True if URL is being removed from the blacklist, false if it's being added to it.
 */
function setUrlPermissions(url, value) {
    let notAllowedUrls = JSON.parse(localStorage.getItem(URLS_BLACKLIST_KEY));
    if (value) {
        delete notAllowedUrls[url]
    } else {
        notAllowedUrls[url] = true;
    }

    localStorage.setItem(URLS_BLACKLIST_KEY, JSON.stringify(notAllowedUrls));
}

/**
 * Extracts the host name from the provided URL.
 *
 * @param url - URL to extract host name from.
 * @returns {string | *} - Extracted host name.
 */
function extractHostname(url) {
    let hostname;
    //find & remove protocol (http, ftp, etc.) and get hostname

    if (url.indexOf("://") > -1) {
        hostname = url.split('/')[2];
    } else {
        hostname = url.split('/')[0];
    }

    //find & remove port number
    hostname = hostname.split(':')[0];
    //find & remove "?"
    hostname = hostname.split('?')[0];

    return hostname;
}

/**
 * Extract the root domain from the provided URL.
 *
 * @param url - URL to extract the root domain from.
 * @returns {*} - The root domain.
 */
function extractRootDomain(url) {
    var domain = extractHostname(url),
        splitArr = domain.split('.'),
        arrLen = splitArr.length;

    //extracting the root domain here
    //if there is a subdomain
    if (arrLen > 2) {
        domain = splitArr[arrLen - 2] + '.' + splitArr[arrLen - 1];
        //check to see if it's using a Country Code Top Level Domain (ccTLD) (i.e. ".me.uk")
        if (splitArr[arrLen - 2].length == 2 && splitArr[arrLen - 1].length == 2) {
            //this is using a ccTLD
            domain = splitArr[arrLen - 3] + '.' + domain;
        }
    }
    return domain;
}

function roReady() {
    return new Promise((resolve, reject) => {
        isRoReady = true;
    });
}
